/******************************************************************************
 *
 * FILE: gtm.c
 *
 * Description:
 *    PWM generation with use of the GTM module
 *
 * History:
 *    15.12.2017   PN   Initial version
 *    09.02.2018   MF   version for SPC57xx, GTM122
 *
 ******************************************************************************
 * Copyright (C) 2015-2017 HighTec EDV-Systeme GmbH.  All rights reserved.
 *
 * This is proprietary software. Any use of the software requires a written
 * license agreement with HighTec EDV-Systeme GmbH. Please contact for
 * detailed license information: info@hightec-rt.com.
 *****************************************************************************/

 /* BSP support package */
#include "bsp.h"

/* GTM timer header */
#include "gtm.h"

uint32_t g_gtm_tom0ch0_pwmDuty = 1;
uint8_t g_gtm_tom0ch0_direction = 1;

static void gtm_SetDuty(uint32_t duty);

/* ============================================================================
 *            ISR FUNCTIONS
 * ==========================================================================*/

/* ----------------------------------------------------------------------------
 * FUNCTION: isr_tom1_RFDF()
 *    TOM1 overflow ISR Handler
 * --------------------------------------------------------------------------*/

void Tom0Ch1IsrHandler(void)
{
    uint32_t pwmDuty = g_gtm_tom0ch0_pwmDuty;
    uint8_t direction = g_gtm_tom0ch0_direction;

    /* Global variable check */
    if((pwmDuty < 1)||(pwmDuty > 99))
    {
    	pwmDuty = (uint32_t) 50; /* If g_gtm_tom0ch1_pwmDuty is out of the
    	                            range, default is 50% duty cycle*/
    }
    if(direction > 1)
    {
    	direction = (uint8_t) 0; /* If g_gtm_tom0ch1_direction is out of the
    	                            range, default is 0 increase duty */
    }

	/* Sweep PWM duty from 1% - 99% */
	/* increase duty */
	if (direction == 1) {
		if (pwmDuty < 99){
			pwmDuty++;
		}
		else{
			direction = 0;
		}
	}
	/* decrease duty */
	else{
		if (pwmDuty > 1){
			pwmDuty--;
		}
		else{
			direction = 1;
		}
	}

	/* Update PWM duty */
	gtm_SetDuty(pwmDuty);

	g_gtm_tom0ch0_pwmDuty = pwmDuty;
	g_gtm_tom0ch0_direction = direction;

	GTM.TOM[0].CH[1].IRQ_NOTIFY.B.CCU0TC = 1; 		/* Clear interrupt flag */
}

/* ============================================================================
 * GTM example API
 * ==========================================================================*/

/* ----------------------------------------------------------------------------
 * Name:        gtm_Init
 * Parameters:
 *              none
 * Returns:
 *              none
 * Description: Initialization of GTM CMU - Global clock GCLK and FXCLK
 *				Initialization of PIN for Tom0_1 and Tom0_0
 * --------------------------------------------------------------------------*/

void gtm_Init(void)
{ 
	/* Configure GTM */
	GTMINT.MCR.R = 0;	/* Enable GTM module, AEI soft-reset control is
	                       disabled, normal mode */
	GTM.CTRL.U = 0;	/* SW reset enabled, mode: Observe, AEI  timeout
	                   functionality is disabled */
	GTM.RST.U = 0x00000001;	/* Initiate reset action for all submodules */

	/* Init CMU 
	 * setup global clock divider
	 * Tcm_gclk_en = (CMU_GCLK_NUM / CMU_GCLK_DEN) * Tsys_clk */

	/* setup global clock divider - CMU_GCLK_EN = GTM_CLK / 2 */
	GTM.CMU.GCLK_NUM.U 	= GTM_CMU_CFGU_GLOBAL_CLK_DIV;
	GTM.CMU.GCLK_DEN.U 	= 0x1;

	/* enable the clock pre-scalers */
	GTM.CMU.FXCLK_CTRL.B.FXCLK_SEL = 0; /* CMU_GCLK_EN selected - Input clock
	                                       selection for EN_FXCLK line */
	GTM.CMU.CLK_EN.U = 0x00800000; /* Enable FXCLK clock signal */

	/* Configure TOM output pins */
	/* TOM0_channel0 port configuration PB[9] */
	/* enable output buffer, default (weak) drive, TOM0_0 function */
	bsp_pin_EnableOutput(GTN_PIN_PB9, GTM_TOM0_0);
	/* TOM0_channel1 port configuration PB[10] */
	/* enable output buffer, medium drive, TOM0_1 function */
	bsp_pin_EnableOutput(GTN_PIN_PB10, GTM_TOM0_1);
}

/* ----------------------------------------------------------------------------
 * Name:        gtm_InitTom0
 * Parameters:
 *              none
 * Returns:
 *              none
 * Description:
 *              Function for TOM module initialization
 * --------------------------------------------------------------------------*/

void gtm_InitTom0 (void)
{
	/* TOM global control configuration
	 * Enable update  of  register  CM0,  CM1 and CLK_SRC_STAT from SR0, SR1
	 * and CLK_SRC, no reset action after write by CPU, no trigger request */
	GTM.TOM[0].TGC[0].GLB_CTRL.U = 0x000A0000;
	GTM.TOM[0].TGC[0].ACT_TB.U = 0x00000000;	/* TBU0_TS0 selected */

	/* TOM0 channel 0 configuration
	 * SL=1, GTM_TOM0_CH0_CTRL_CLK_SRC_SR as a source clock, Reset CN0 to 0 on
	 * matching comparison CM0, TRIG_CCU0, One-shot mode disabled, SPE mode
	 * disabled, Gated Counter mode disabled */
	GTM.TOM[0].CH[0].CTRL.U = \
			(uint32_t)(0x01000800 | (GTM_TOM0_CH0_CTRL_CLK_SRC_SR << 12));

	/* set period: (FXCLK / period) */
	GTM.TOM[0].CH[0].SR0.U = (uint16_t)GTM_TOM0_CH0_PERIODE_TICKS;
	/* set duty cycle: (period * duty) / 100 */
	GTM.TOM[0].CH[0].SR1.U = (uint16_t)(GTM_TOM0_CH0_PERIODE_TICKS/2);
	/* set period*/
	GTM.TOM[0].CH[0].CM0.U = (uint16_t)GTM_TOM0_CH0_PERIODE_TICKS;
	/* set duty cycle */
	GTM.TOM[0].CH[0].CM1.U = (uint16_t)(GTM_TOM0_CH0_PERIODE_TICKS/2);
	GTM.TOM[0].CH[0].CN0.U = (uint16_t)GTM_TOM0_CH0_PERIODE_TICKS;

	 /* TOM0 channel 1 configuration 
	  * SL=1, GTM_TOM0_CH1_CTRL_CLK_SRC_SR as a source clock, Reset CN0 to 0 on
	  * matching comparison CM0, TRIG_CCU0, One-shot mode disabled, SPE mode
	  * disabled, Gated Counter mode disabled */
	GTM.TOM[0].CH[1].CTRL.U = \
			(uint32_t)(0x01000800 | (GTM_TOM0_CH1_CTRL_CLK_SRC_SR << 12));

	/* set period - (FXCLK / period) */
	GTM.TOM[0].CH[1].SR0.U  = (uint16_t)GTM_TOM0_CH1_PERIODE_TICKS;
	/* set duty cycle -(period * duty) / 100 */
	GTM.TOM[0].CH[1].SR1.U  = \
			(uint16_t)((GTM_TOM0_CH1_PERIODE_TICKS * GTM_TOM0_CH1_DUTY)/100);

	/* set period */
	GTM.TOM[0].CH[1].CM0.U  = (uint16_t)GTM_TOM0_CH1_PERIODE_TICKS;
	/* set duty cycle */
	GTM.TOM[0].CH[1].CM1.U  = \
			(uint16_t)((GTM_TOM0_CH1_PERIODE_TICKS * GTM_TOM0_CH1_DUTY)/100);
	GTM.TOM[0].CH[1].CN0.U = GTM_TOM0_CH1_PERIODE_TICKS;

	/* enable counter CCU0TC interrupt */
	GTM.TOM[0].CH[1].IRQ_EN.B.CCU0TC_IRQ_EN = 1;

	/* Register interrupt handler */
	bsp_isr_RegisterHandler(UC_GTM_TOM0_CH1_ISR, 10, Tom0Ch1IsrHandler);

	/* enable TOM0 Ch0,1 (start counters) */
	GTM.TOM[0].TGC[0].ENDIS_STAT.U = 0x0000000A;
	/* enable output for TOM0 Ch0,1 */
	GTM.TOM[0].TGC[0].OUTEN_STAT.U= 0x0000000A;
}

static void gtm_SetDuty(uint32_t duty)
{
	uint32_t tmpPeriod;
	uint32_t tmpDuty;

	/* Prepare period and duty constant */
	tmpPeriod  = GTM.TOM[0].CH[0].SR0.U;
	tmpDuty = (tmpPeriod * duty) / 100;

	GTM.TOM[0].CH[0].SR1.U  = (uint16_t)tmpDuty;	/* set duty cycle */

	
}
