/******************************************************************************
 *
 * FILE: gtm.h
 *
 * Description:
 *    PWM generation with use of the GTM timer
 *
 * History:
 *    15.12.2017   PN   Initial version
 *    12.02.2018   MF   Update for SPC574K72
 *
 ******************************************************************************
 * Copyright (C) 2015-2017 HighTec EDV-Systeme GmbH.  All rights reserved.
 *
 * This is proprietary software. Any use of the software requires a written
 * license agreement with HighTec EDV-Systeme GmbH. Please contact for
 * detailed license information: info@hightec-rt.com.
 *****************************************************************************/

#ifndef GTM_H
#define GTM_H

/* ============================================================================
 * GTM EXAMPLE SETTING
 * User ** can ** change it, here in this file
 * ==========================================================================*/

/* GTM CMU Global clock */
#define GTM_CMU_GCLK_EN                   10	    /* 10MHz */

/* GTM TOM0 period
 * Note: Example changes duty-cycle in time
 * */
#define GTM_TOM0_CH0_PERIODE_TIME_US        20      /* us */

/*GTM TOM0_CH1 period and duty-cycle, generating the interrupt */
#define GTM_TOM0_CH1_PERIODE_TIME_US        50000  /* us */
#define GTM_TOM0_CH1_DUTY                   50     /* % */

/* ============================================================================
 * GTM, PIN & CLOCK INSTANCES IMPLEMENTATION
 * Only relevant configuration for BSP support package
 * ==========================================================================*/

#define GTM_TOM0_0	8	/* Port pin functionality */
#define GTM_TOM0_1	8	/* Port pin functionality */

/* TOM0 ports */
#define GTN_PIN_PB9		25
#define GTN_PIN_PB10	26

/* GTM Moudule clock from AUX */
#define GTM_CMU_SYS_CLOCK           UC_GTM_CLK/2

/* GTM TOM0 Channel 1 ISR Vector (base vector for subsequent channels)  */
#define UC_GTM_TOM0_CH1_ISR		    830

/* ============================================================================
 * GTM EXTERN
 * ==========================================================================*/

extern uint32_t g_gtm_tom0ch0_pwmDuty;  /* Duty cycle, range 1 to 99%*/
extern uint8_t g_gtm_tom0ch0_direction; /* Increasing or decreasing the duty
                                           cycle ( 1 or 0  ) */

/* ============================================================================
 * GTM API INTERFACE
 * ==========================================================================*/

void gtm_Init(void);
void gtm_InitTom0 (void);

/* ============================================================================
 * GTM CONFIGURATION
 * Example GTM setting
 * ==========================================================================*/
#define GTM_CMU_CFGU_GLOBAL_CLK_DIV     (GTM_CMU_SYS_CLOCK/GTM_CMU_GCLK_EN)

/* TOM0 CH0 Clock source select for channel */
#if((GTM_TOM0_CH0_PERIODE_TIME_US * GTM_CMU_GCLK_EN) < 0x10000U)
#define GTM_TOM0_CH0_CTRL_CLK_SRC_SR    0 /* clock selected by FXCLK_SEL */
#elif(((GTM_TOM0_CH0_PERIODE_TIME_US * GTM_CMU_GCLK_EN)/16) < 0x10000U)
#define GTM_TOM0_CH0_CTRL_CLK_SRC_SR    1 /* clock selected by FXCLK_SEL/2^4 */
#elif(((GTM_TOM0_CH0_PERIODE_TIME_US * GTM_CMU_GCLK_EN)/256) < 0x10000U)
#define GTM_TOM0_CH0_CTRL_CLK_SRC_SR    2 /* clock selected by FXCLK_SEL/2^8 */
#elif(((GTM_TOM0_CH0_PERIODE_TIME_US * GTM_CMU_GCLK_EN)/4096) < 0x10000U)
#define GTM_TOM0_CH0_CTRL_CLK_SRC_SR    3 /* clock selected by FXCLK_SEL/2^12*/
#elif(((GTM_TOM0_CH0_PERIODE_TIME_US * GTM_CMU_GCLK_EN)/65536) < 0x10000U)
#define GTM_TOM0_CH0_CTRL_CLK_SRC_SR    4 /* clock selected by FXCLK_SEL/2^16*/
#else
#error TOM0 CH0 periode is out of range
#endif

/* TOM0 CH1 Clock source select for channel */
#if((GTM_TOM0_CH1_PERIODE_TIME_US * GTM_CMU_GCLK_EN) < 0x10000U)
#define GTM_TOM0_CH1_CTRL_CLK_SRC_SR    0 /* clock selected by FXCLK_SEL */
#elif(((GTM_TOM0_CH1_PERIODE_TIME_US * GTM_CMU_GCLK_EN)/16) < 0x10000U)
#define GTM_TOM0_CH1_CTRL_CLK_SRC_SR    1 /* clock selected by FXCLK_SEL/2^4 */
#elif(((GTM_TOM0_CH1_PERIODE_TIME_US * GTM_CMU_GCLK_EN)/256) < 0x10000U)
#define GTM_TOM0_CH1_CTRL_CLK_SRC_SR    2 /* clock selected by FXCLK_SEL/2^8 */
#elif(((GTM_TOM0_CH1_PERIODE_TIME_US * GTM_CMU_GCLK_EN)/4096) < 0x10000U)
#define GTM_TOM0_CH1_CTRL_CLK_SRC_SR    3 /* clock selected by FXCLK_SEL/2^12*/
#elif(((GTM_TOM0_CH1_PERIODE_TIME_US * GTM_CMU_GCLK_EN)/65536) < 0x10000U)
#define GTM_TOM0_CH1_CTRL_CLK_SRC_SR    4 /* clock selected by FXCLK_SEL/2^16*/
#else
#error TOM0 CH1 periode is out of range
#endif

#define GTM_TOM0_CH0_PERIODE_TICKS \
	(uint16_t)((GTM_TOM0_CH0_PERIODE_TIME_US * GTM_CMU_GCLK_EN)/ \
			   (1<<(4*GTM_TOM0_CH0_CTRL_CLK_SRC_SR)))
#define GTM_TOM0_CH1_PERIODE_TICKS \
	(uint16_t)((GTM_TOM0_CH1_PERIODE_TIME_US * GTM_CMU_GCLK_EN)/ \
			   (1<<(4*GTM_TOM0_CH1_CTRL_CLK_SRC_SR)))

/* ============================================================================
 * GTM CONFIGURATION CHECKS
 * ==========================================================================*/

/* Duty cycle for TOM0 CH1 - <1% to 99%> */
#if ((GTM_TOM0_CH1_DUTY < 1) || (GTM_TOM0_CH1_DUTY > 99))
#error Duty cycle is out of the range 1 to 99%
#endif

/* Protection against the division by zero*/
#if (GTM_CMU_CFGU_GLOBAL_CLK_DIV < 1)
#error CMU_GCLK_DEN, the Denominator should be 1 or higher
#endif

/* FXCLK whole integer divider check */
#if ((GTM_CMU_SYS_CLOCK) - \
	 (GTM_CMU_GCLK_EN * GTM_CMU_CFGU_GLOBAL_CLK_DIV))
#error FXCLK clock is not exact, divider is not a whole number. Check the setting
#endif

#endif /* GTM_H */
