/******************************************************************************
 * FILE
 *    bsp.h
 *
 * UC ARCHITECTURE:
 *    PowerPC
 *
 * UC FAMILY:
 *    SPC58
 *
 * DESCRIPTION:
 *    BSP package HW abstraction layer (HAL) API interface.
 *
 *    Main goals of BSP HAL
 *       1. to initialize uC platform in a optimal configuration for selected
 *          clock resources and targets
 *       2. to provide set of user friendly functions simplifying first steps
 *          in uC evaluation
 *
 *    Only few selected HW modules are used to fulfill goals!
 *    Other modules can be added as application extension.
 *
 *    BSP HAL configuration
 *       Certain functionality can be switched OFF/ON through BSP defines
 *
 *       BSP_HW_INIT_CHECK: 0/1 = OFF/ON (default ON)
 *          uC configuration internal checks that control correctness
 *          of user clock configuration. In case user provides his own
 *          platform initialization, this check shall be switched OFF.
 *
 * FILE HISTORY:
 *    V1.0    23.09.2017     RO
 *       > Initial version
 *    V2.0    25.01.2018     RO
 *       > File name rename and file header update
 *       > bsp_crt0_PreInit and PostInit functions added to simplify
 *         application crt0 hooks implementation
 *       > BSP_TIMER_SUPPORT configuration option added
 *       > BSP_CORE_ENABLE configuration option added
 *
 *
 ******************************************************************************
 * Copyright (C) 2015-2018 HighTec EDV-Systeme GmbH.  All rights reserved.
 * This is proprietary software. Any use of the software requires a written
 * license agreement with HighTec EDV-Systeme GmbH. Please contact for
 * detailed license information: info@hightec-rt.com.
 *****************************************************************************/

#ifndef BSP_H
#define BSP_H

/* Include common typedefs */
#include "typedefs.h"


/* ============================================================================
 * uC DERIVATIVE USER CONFIGURATION
 * ==========================================================================*/

#if defined(BSP_USER_CONFIG)
#include "bsp_user_config.h"
#endif


/* ============================================================================
 * BSP DEFAULT CONFIGURATION
 * Might be changed from outside by defining the same symbols with
 * a different values
 * ==========================================================================*/

/* BSP HW Initialization control
 * 0 = OFF, no checks of correct UC configuration values
 * 1 = ON (default), uC spec checks group of parameters for their
 *                   correct HW setting */
#ifndef BSP_HW_INIT_CHECK
#define BSP_HW_INIT_CHECK         1
#endif

/* BSP_CORE_ENABLE: Inactive Core Enable control
 * 0 = OFF, inactive cores will not be enabled by bsp_crt0_PostInit()
 * 1 = ON (default), inactive cores will be enabled by bsp_crt0_PostInit() */
#ifndef BSP_CORE_ENABLE
#define BSP_CORE_ENABLE           1
#endif

/* BSP_ISR_TABLE: BSP implementation of ISR table in RAM
 * 0 = OFF, BSP will not implement its bsp_isr_table[1024] in RAM
 *          user must provide his own ISR table (either ROM or RAM)
 * 1 = ON (default), BSP implements its bsp_isr_table[1024] in RAM */
#ifndef BSP_ISR_TABLE
#define BSP_ISR_TABLE             1
#endif

/* BSP_PIT_SUPPORT: BSP PIT timer control enable/disable
 * 0 = OFF, PIT timer will not be initialized and PIT interface disabled
 * 1 = ON (default), PIT timer under BSP control */
#ifndef BSP_TIMER_SUPPORT
#define BSP_TIMER_SUPPORT         1
#endif

/* SYSCLK0 Output Enable Control
 * 0 = OFF, switch off clock generation setting
 * 1 = ON, BSP will enable clock generation output */
#ifndef BSP_SYSCLK0_OUT
#define BSP_SYSCLK0_OUT           1
#endif

/* SYSCLK1 Output Enable Control
 * 0 = OFF, switch off clock generation setting
 * 1 = ON, BSP will enable clock generation output */
#ifndef BSP_SYSCLK1_OUT
#define BSP_SYSCLK1_OUT           1
#endif

/* BSP_ISR_MODE: Interrupt Vector mode
 * 0 = SW, Software vector mode using crt0 IVOR4 SW Handler
 * 1 = HW, Using direct call of ISR Handler
 *         (INTERRUPT_FXN and BSP_EXIT_INTERRUPT macros are needed)
 */
#define BSP_ISR_SW_MODE          0
#define BSP_ISR_HW_MODE          1


/* ============================================================================
 * uC DERIVATIVE IMPLEMENTATION HEADERS
 * ==========================================================================*/

/* Include uC derivative implementation header file */
#include "../uc/uc.h"

/* Include spc58xx LLD driver interface used by HHL BSP API */
#include "uc_lld/uc_lld.h"


/* ============================================================================
 * BSP TYPES
 * Application will needed them when using BSP API
 * ==========================================================================*/

/* Function pointer type for ISR Vector Table registration */
typedef void(*IsrHandler)(void);


/* ============================================================================
 * BSP PUBLIC SYMBOLS
 * Public symbols declaration
 * ==========================================================================*/


/* ============================================================================
 * BSP CONSTANTS
 * Those that might bring benefit to the user
 * ==========================================================================*/

/* TICK definitions for timer constants - expects to have PER_CLK >= 1MHz*/
#define BSP_TICKS_1US        (UC_PER_CLK0)            /* ticks to get 1us */
#define BSP_TICKS_1MS        (BSP_TICKS_1US * 1000)   /* ticks to get 1ms */


/* ============================================================================
 * BSP API INTERFACE
 * One common BSP HLL interface to simplify user startup with uC architecture.
 * ==========================================================================*/

/* crt0 hooks */
void bsp_crt0_PreInit(uint32_t coreId);
void bsp_crt0_PostInit(uint32_t coreId);

/* uC Platform */
void bsp_uc_InitPlatform();
void bsp_uc_InitCore();
void bsp_uc_StartCore(uint32_t core_nb);
void bsp_uc_StartAllCores(void);
void bsp_uc_RegisterIsrTable(uint32_t isr_mode, uint32_t* isr_table);
uint32_t bsp_uc_GetCurrentCoreId(void);

/* IO PORTS */
void bsp_pin_EnableInput(uint32_t pin_nb);
void bsp_pin_EnableOutput(uint32_t pin_nb, uint32_t source);
void bsp_pin_EnableBidir(uint32_t pin_nb, uint32_t source);
void bsp_pin_SetOutput(uint32_t pin_nb, uint8_t value);
void bsp_pin_Toggle(uint32_t pin_nb);
uint8_t bsp_pin_GetInput(uint32_t pin_nb);

/* INTERRUPTS */
void bsp_isr_Init(void);
void bsp_isr_RegisterHandler(uint32_t vector_nb, uint32_t priority, IsrHandler handler);
void bsp_isr_EnableExternalInterrupts(void);
void bsp_isr_DisableExternalInterrupts(void);
void bsp_isr_ExitHWInterrupt(void);

/* TIMER */
#if (BSP_TIMER_SUPPORT)
void bsp_timer_EnableModule(void);
void bsp_timer_ReloadChannel(UC_PIT_CHANNEL_e channel, uint32_t period);
void bsp_timer_EnableChannelIsr(UC_PIT_CHANNEL_e channel, uint32_t priority, IsrHandler handler);
void bsp_timer_StartChannel(UC_PIT_CHANNEL_e channel);
void bsp_timer_ClearChannelIsrFlag(UC_PIT_CHANNEL_e channel);
uint32_t bsp_timer_StopChannel(UC_PIT_CHANNEL_e channel);
uint32_t bsp_timer_IsChannelIsrFlagActive(UC_PIT_CHANNEL_e channel);
#endif /* BSP_TIMER_SUPPORT */

/* SYSTEM CLOCKOUT */
#if (UC_MODULE_SYSCLK0)
void bsp_clk_EnableSYSCLK0Output(void);
#endif
#if (UC_MODULE_SYSCLK1)
void bsp_clk_EnableSYSCLK1Output(void);
#endif

#endif /* BSP_H */
