/******************************************************************************
 * FILE: 
 *    spc58nn84.ld
 *
 * UC ARCHITECTURE:
 *    PowerPC
 *
 * UC FAMILY:
 *    SPC58
 *
 * DESCRIPTION:
 *    SPC58NN84 microcontroller linker file template
 *
 * History:
 *    V1.0    23.09.2017   RO   
 *       - Initial version
 *
 ******************************************************************************
 * Copyright (C) 2015-2018 HighTec EDV-Systeme GmbH.  All rights reserved.
 * This is proprietary software. Any use of the software requires a written
 * license agreement with HighTec EDV-Systeme GmbH. Please contact for
 * detailed license information: info@hightec-rt.com.
 *****************************************************************************/


/* ============================================================================
 * SYMBOLS
 * ==========================================================================*/

/* Stack size definition */
__STACK_SIZE_CORE0 = 4K;
__STACK_SIZE_CORE1 = 4K;
__STACK_SIZE_CORE2 = 4K;

/* Entry point for debugger in output elf file */
ENTRY(_crt0_reset)


/* ============================================================================
 * uC MEMORY REGIONS
 * ==========================================================================*/

MEMORY
{
    /* FLASH memories */
    FLASH          (rx)  : org = 0x00FC0000, len = 0x600000  /* 6MB (both Flash modules as one) */
    
    /* global RAM memories */
    SRAM0          (rw)  : org = 0x40070000, len = 0x10000   /* 64kB */
    SRAM2          (rw)  : org = 0x40080000, len = 0x10000   /* 64kB */
    
    /* CORE Local memories */
    IMEM_CORE0     (rwx) : org = 0x50000000, len =  0x8000   /*  32kB Instruction RAM */
    DMEM_CORE0     (rw)  : org = 0x50800000, len = 0x20000   /* 128kB Data RAM */
    IMEM_CORE1     (rwx) : org = 0x51000000, len =  0x8000   /*  32kB Instruction RAM */
    DMEM_CORE1     (rw)  : org = 0x51800000, len = 0x20000   /* 128kB Data RAM */
    IMEM_CORE2     (rwx) : org = 0x52000000, len =  0x8000   /*  32kB Instruction RAM */
    DMEM_CORE2     (rw)  : org = 0x52800000, len = 0x20000   /* 128kB Data RAM */
}


/* ============================================================================
 * SECTION CONTAINERS TO MEMORY REGION MAPPING
 * ==========================================================================*/

/* CRT0 header and sections */ 
REGION_ALIAS("BOOT", FLASH)

/* IVOR & ISR vectors */
REGION_ALIAS("IVOR", FLASH) 
REGION_ALIAS("ISR", SRAM2) 

/* CORE(0) code, ram_code, const and data */
REGION_ALIAS("CODE_CPU0_",     FLASH)
REGION_ALIAS("RODATA_CPU0_",   FLASH)
REGION_ALIAS("STACK_CPU0_",    DMEM_CORE0) 
REGION_ALIAS("DATA_CPU0_",     DMEM_CORE0)
REGION_ALIAS("RAM_CODE_CPU0_", IMEM_CORE0)

/* CORE(1) code, ram_code, const and data */
REGION_ALIAS("CODE_CPU1_",     FLASH)
REGION_ALIAS("RODATA_CPU1_",   FLASH)
REGION_ALIAS("STACK_CPU1_",    DMEM_CORE1) 
REGION_ALIAS("DATA_CPU1_",     DMEM_CORE1)
REGION_ALIAS("RAM_CODE_CPU1_", IMEM_CORE1)

/* CORE(2) code, ram_code, const and data */
REGION_ALIAS("CODE_CPU2_",     FLASH)
REGION_ALIAS("RODATA_CPU2_",   FLASH)
REGION_ALIAS("STACK_CPU2_",    DMEM_CORE2) 
REGION_ALIAS("DATA_CPU2_",     DMEM_CORE2)
REGION_ALIAS("RAM_CODE_CPU2_", IMEM_CORE2)

/* COMMON code, const and data */
REGION_ALIAS("CODE",           FLASH)
REGION_ALIAS("RODATA",         FLASH)
REGION_ALIAS("DATA",           SRAM2)
REGION_ALIAS("RAM_CODE",       SRAM2)


/* ============================================================================
 * SECTION GROUPS (INPUT -> OUTPUT)
 * ==========================================================================*/

SECTIONS
{
	/* ------------------------------------------------------------------------
     * CRT0 BOOT 
     * rchw: Boot Header searched for BAM at predefined addresses in flash
     * crt0_asm: 'C' runtime environment init
     * ----------------------------------------------------------------------*/
	
	/* RCHW Reset word */
    .rchw : FLAGS(axvl)    {
        KEEP(*(.rchw))
    } > BOOT

	/* CRT0 startup code */
    .crt0 : ALIGN(8) FLAGS(avxl)
    {
        KEEP(*(.crt0))
    } > BOOT

	/* ------------------------------------------------------------------------
     * IVOR & ISR TABLES
     * ivor_table: exception table with calls to exception handlers 
     * ivor_handlers: crt0 provided exception handlers
     * isrTable: vector table with interrupt handler entry points
     * ----------------------------------------------------------------------*/

	.ivor_table : ALIGN(256) FLAGS(avxl)
	{
        KEEP(*(.ivor_table))
	} > IVOR

	.ivor_handlers : FLAGS(avxl)
	{
        *(.ivor_handlers)
	} > IVOR

	/* ------------------------------------------------------------------------
     * CORE(x) STACK MEMORY
     * We create one Stack for each Core
     * Size is defined at the beginning of the linker file
     * ----------------------------------------------------------------------*/
	
    .CPU0.stack  : ALIGN(8) FLAGS(aw)
    {
        __SP_END_CORE0 = .;
        . +=  __STACK_SIZE_CORE0;
        __SP_INIT_CORE0 = .;
    } > STACK_CPU0_

    .CPU1.stack  : ALIGN(8) FLAGS(aw)
    {
        __SP_END_CORE1 = .;
        . +=  __STACK_SIZE_CORE1;
        __SP_INIT_CORE1 = .;
    } > STACK_CPU1_

    .CPU2.stack  : ALIGN(8) FLAGS(aw)
    {
        __SP_END_CORE2 = .;
        . +=  __STACK_SIZE_CORE2;
        __SP_INIT_CORE2 = .;
    } > STACK_CPU2_

	/* ------------------------------------------------------------------------
     * CORE(0) specific sections
	 * all that have special naming of sections starting with prefix .CPU0
     * ----------------------------------------------------------------------*/

	/* Code collection */
    .CPU0.code : FLAGS(axv)
    {  
        /* USER INPUT SECTION SELECTION PATTERN FOR CORE0    */
        /* .text_vle sections                                */
    } > CODE_CPU0_

	/* Code to run from IMEM */
    .CPU0.ram_code : FLAGS(axv)
    {  
        /* USER INPUT SECTION SELECTION PATTERN FOR CORE0    */
        /* .ram_code sections                                */
    } > RAM_CODE_CPU0_ AT > RODATA_CPU0_
	
	/* CONST DATA in Flash */
    .CPU0.rodata : ALIGN(8) FLAGS(arl)
    {  
        /* USER INPUT SECTION SELECTION PATTERN FOR CORE0    */
        /* .rodata sections                                  */
    } > RODATA_CPU0_

	/* Init data in RAM */
    .CPU0.data : FLAGS(awl)
    {  
        /* USER INPUT SECTION SELECTION PATTERN FOR CORE0    */
        /* .data sections                                    */
    } > DATA_CPU0_ AT > RODATA_CPU0_

	/* Non-init data in RAM */
	.CPU0.bss : FLAGS(awl)
    {  
        /* USER INPUT SECTION SELECTION PATTERN FOR CORE0    */
        /* .bss sections                                     */
    } > DATA_CPU0_

	/* ------------------------------------------------------------------------
     * CORE(1) specific sections
	 * all that have special naming of sections starting with prefix .CPU1
     * ----------------------------------------------------------------------*/

	/* Code collection */
    .CPU1.code : FLAGS(axv)
    {  
        /* USER INPUT SECTION SELECTION PATTERN FOR CORE1    */
        /* text_vle sections                                 */
    } > CODE_CPU1_
	
	/* Code to run from IMEM */
    .CPU1.ram_code : FLAGS(axv)
    {  
        /* USER INPUT SECTION SELECTION PATTERN FOR CORE0    */
        /* .ram_code sections                                */
    } > RAM_CODE_CPU1_ AT > RODATA_CPU1_
	
	/* CONST DATA in Flash */
    .CPU1.rodata : ALIGN(8) FLAGS(arl)
    {  
        /* USER INPUT SECTION SELECTION PATTERN FOR CORE1    */
        /* .rodata sections                                  */
    } > RODATA_CPU1_

	/* Init data in RAM */
    .CPU1.data : FLAGS(awl)
    {  
        /* USER INPUT SECTION SELECTION PATTERN FOR CORE1    */
        /* .data sections                                    */
    } > DATA_CPU1_ AT > RODATA_CPU1_

	/* Init data in RAM */
    .CPU1.bss : FLAGS(awl)
    {  
        /* USER INPUT SECTION SELECTION PATTERN FOR CORE1 */
        /* .bss sections                                  */
    } > DATA_CPU1_

	/* ------------------------------------------------------------------------
     * CORE(2) specific sections
	 * all that have special naming of sections starting with prefix .CPU2
     * ----------------------------------------------------------------------*/

	/* Code collection */
    .CPU2.code : FLAGS(axv)
    {  
        /* USER INPUT SECTION SELECTION PATTERN FOR CORE2    */
        /* text_vle sections                                 */
    } > CODE_CPU2_
	
	/* Code to run from IMEM */
    .CPU2.ram_code : FLAGS(axv)
    {  
        /* USER INPUT SECTION SELECTION PATTERN FOR CORE0    */
        /* .ram_code sections                                */
    } > RAM_CODE_CPU2_ AT > RODATA_CPU2_
	
	/* CONST DATA in Flash */
    .CPU2.rodata : ALIGN(8) FLAGS(arl)
    {  
        /* USER INPUT SECTION SELECTION PATTERN FOR CORE2    */
        /* .rodata sections                                  */
    } > RODATA_CPU2_

	/* Init data in RAM */
    .CPU2.data : FLAGS(awl)
    {  
        /* USER INPUT SECTION SELECTION PATTERN FOR CORE2    */
        /* .data sections                                    */
    } > DATA_CPU2_ AT > RODATA_CPU2_

	/* Init data in RAM */
    .CPU2.bss : FLAGS(awl)
    {  
        /* USER INPUT SECTION SELECTION PATTERN FOR CORE2 */
        /* .bss sections                                  */
    } > DATA_CPU2_

	/* ------------------------------------------------------------------------
     * COMMON SECTIONS
	 * all that is not assigned to specific Core in above section mapping
     * ----------------------------------------------------------------------*/

	/* Code collection */
    .code : FLAGS(axv)
    {  
        *(.text_vle*)
        *(*.text_vle*)
    } > CODE
	
	/* Code to run from SRAM */
    .ram_code : FLAGS(axv)
    {  
        *(.ramcode*)
        *(*.ramcode*)
    } > RAM_CODE AT > RODATA
	
	/* CONST DATA in Flash */
    .rodata : ALIGN(8) FLAGS(arl)
    {  
        *(.rodata*)
        *(*.rodata*)
    } > RODATA

	/* Init data in RAM */
    .data : FLAGS(awl)
    {  
        *(.data*)
        *(*.data*)
    } > DATA AT > RODATA

	/* Init data in RAM */
    .bss : FLAGS(awl)
    {  
        *(.bss*)
        *(*.bss*)
    } > DATA

	/* ------------------------------------------------------------------------
     * CLEAR & COPY TABLES
     * clear_table: data memory ranges to clear to zero
     * copy_table: data memory ranges that needs to be initialized with values
     *             init values are stored in FLASH and copied to RAM
     * ----------------------------------------------------------------------*/

	/* CORE(0) - clear data table */
	.CPU0.clear_table : ALIGN(8) FLAGS(arl)
	{
		PROVIDE(__clear_table_core0 = .);
		LONG(ORIGIN(IMEM_CORE0))    ; LONG(LENGTH(IMEM_CORE0));
		LONG(ADDR(.CPU0.bss))       ; LONG(SIZEOF(.CPU0.bss));
		LONG(-1)                    ; LONG(-1);
	} > RODATA_CPU0_

	/* CORE(0) - init data table */
    .CPU0.copy_table  : ALIGN(8) FLAGS(arl)
    {
        PROVIDE(__copy_table_core0 = .);
        LONG(LOADADDR(.CPU0.ram_code)) ; LONG(0 + ADDR(.CPU0.ram_code)) ; LONG(SIZEOF(.CPU0.ram_code));
        LONG(LOADADDR(.CPU0.data))     ; LONG(0 + ADDR(.CPU0.data))     ; LONG(SIZEOF(.CPU0.data));
        LONG(-1)                       ; LONG(-1)                        ; LONG(-1);
    } > RODATA_CPU0_

	/* CORE(1) - clear data table */
	.CPU1.clear_table : ALIGN(8) FLAGS(arl)
	{
		PROVIDE(__clear_table_core1 = .);
		LONG(ORIGIN(IMEM_CORE1))    ; LONG(LENGTH(IMEM_CORE1));
		LONG(ADDR(.CPU1.bss))       ; LONG(SIZEOF(.CPU1.bss));
		LONG(-1)                    ; LONG(-1);
	} > RODATA_CPU1_

	/* CORE(1) - init data table */
    .CPU1.copy_table  : ALIGN(8) FLAGS(arl)
    {
        PROVIDE(__copy_table_core1 = .);
        LONG(LOADADDR(.CPU1.ram_code)) ; LONG(0 + ADDR(.CPU1.ram_code)) ; LONG(SIZEOF(.CPU1.ram_code));
        LONG(LOADADDR(.CPU1.data))     ; LONG(0 + ADDR(.CPU1.data))     ; LONG(SIZEOF(.CPU1.data));
        LONG(-1)                       ; LONG(-1)                       ; LONG(-1);
    } > RODATA_CPU1_


	/* CORE(2) (IOP) - clear data table */
	.CPU2.clear_table : ALIGN(8) FLAGS(arl)
	{
		PROVIDE(__clear_table_core2 = .);
		LONG(ORIGIN(IMEM_CORE2))     ; LONG(LENGTH(IMEM_CORE2));
		LONG(ADDR(.CPU2.bss))        ; LONG(SIZEOF(.CPU2.bss));
		LONG(ADDR(.bss))             ; LONG(SIZEOF(.bss));
		LONG(-1)                     ; LONG(-1);
	} > RODATA_CPU2_

	/* CORE(2) (IOP) - init data table */
    .CPU2.copy_table  : ALIGN(8) FLAGS(arl)
    {
        PROVIDE(__copy_table_core2 = .);
        LONG(LOADADDR(.CPU2.ram_code))   ; LONG(0 + ADDR(.CPU2.ram_code))   ; LONG(SIZEOF(.CPU2.ram_code));
        LONG(LOADADDR(.CPU2.data))       ; LONG(0 + ADDR(.CPU2.data))       ; LONG(SIZEOF(.CPU2.data));
        LONG(LOADADDR(.ram_code))        ; LONG(0 + ADDR(.ram_code))        ; LONG(SIZEOF(.ram_code));
        LONG(LOADADDR(.data))            ; LONG(0 + ADDR(.data))            ; LONG(SIZEOF(.data));
        LONG(-1)                         ; LONG(-1)                         ; LONG(-1);
    } > RODATA_CPU2_

	/* ------------------------------------------------------------------------
     * CRT0 CONFIG STRUCTURE
     * Initialization structure for uC Cores used in CRT0
     * all values comes from Linker file or from command line
     * ----------------------------------------------------------------------*/

	.rodata : ALIGN(8) FLAGS(arl)
	{
		PROVIDE(__crt0_config = .);    /* base of the config table */
		
		/* Core[0] */
		LONG(__SP_INIT_CORE0);         /* STACK address */ 
		LONG(SIZEOF(.CPU0.stack));     /* STACK size */
		LONG(__clear_table_core0);     /* CLEAR table */
		LONG(__copy_table_core0);      /* COPY table */
		LONG(-1);                      /* SDA pointer - short data */
		LONG(-1);                      /* SDA2 pointer - short const */
		
		/* Core[1] */
		LONG(__SP_INIT_CORE1);         /* STACK address */ 
		LONG(SIZEOF(.CPU1.stack));     /* STACK size */
		LONG(__clear_table_core1);     /* CLEAR table */
		LONG(__copy_table_core1);      /* COPY table */
		LONG(-1);                      /* SDA pointer - short data */
		LONG(-1);                      /* SDA2 pointer - short const */
		
		/* Core[2] */
		LONG(__SP_INIT_CORE2);         /* STACK address */ 
		LONG(SIZEOF(.CPU2.stack));     /* STACK size */
		LONG(__clear_table_core2);     /* CLEAR table */
		LONG(__copy_table_core2);      /* COPY table */
		LONG(-1);                      /* SDA pointer - short data */
		LONG(-1);                      /* SDA2 pointer - short const */
		
	} > RODATA

	/* ------------------------------------------------------------------------
     * OTHER SECTIONS
     * ----------------------------------------------------------------------*/

}


/* ============================================================================
 * OTHER SYMBOLS
 * ==========================================================================*/

/* Extended base registers (linker always searches for these symbols) */
_SDA14_BASE_ = 0x8000;
_SDA15_BASE_ = 0x8000;
_SDA16_BASE_ = 0x8000;
_SDA17_BASE_ = 0x8000;
